# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .photo import (
    PhotoResource,
    AsyncPhotoResource,
    PhotoResourceWithRawResponse,
    AsyncPhotoResourceWithRawResponse,
    PhotoResourceWithStreamingResponse,
    AsyncPhotoResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.whatsapp.phone_numbers import profile_update_params
from .....types.whatsapp.phone_numbers.profile_update_response import ProfileUpdateResponse
from .....types.whatsapp.phone_numbers.profile_retrieve_response import ProfileRetrieveResponse

__all__ = ["ProfileResource", "AsyncProfileResource"]


class ProfileResource(SyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def photo(self) -> PhotoResource:
        """Manage Whatsapp phone numbers"""
        return PhotoResource(self._client)

    @cached_property
    def with_raw_response(self) -> ProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ProfileResourceWithStreamingResponse(self)

    def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProfileRetrieveResponse:
        """
        Get phone number business profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileRetrieveResponse,
        )

    def update(
        self,
        phone_number: str,
        *,
        about: str | Omit = omit,
        address: str | Omit = omit,
        category: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProfileUpdateResponse:
        """
        Update phone number business profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._patch(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile",
            body=maybe_transform(
                {
                    "about": about,
                    "address": address,
                    "category": category,
                    "description": description,
                    "display_name": display_name,
                    "email": email,
                    "website": website,
                },
                profile_update_params.ProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileUpdateResponse,
        )


class AsyncProfileResource(AsyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def photo(self) -> AsyncPhotoResource:
        """Manage Whatsapp phone numbers"""
        return AsyncPhotoResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncProfileResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProfileRetrieveResponse:
        """
        Get phone number business profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileRetrieveResponse,
        )

    async def update(
        self,
        phone_number: str,
        *,
        about: str | Omit = omit,
        address: str | Omit = omit,
        category: str | Omit = omit,
        description: str | Omit = omit,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProfileUpdateResponse:
        """
        Update phone number business profile

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._patch(
            f"/v2/whatsapp/phone_numbers/{phone_number}/profile",
            body=await async_maybe_transform(
                {
                    "about": about,
                    "address": address,
                    "category": category,
                    "description": description,
                    "display_name": display_name,
                    "email": email,
                    "website": website,
                },
                profile_update_params.ProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileUpdateResponse,
        )


class ProfileResourceWithRawResponse:
    def __init__(self, profile: ProfileResource) -> None:
        self._profile = profile

        self.retrieve = to_raw_response_wrapper(
            profile.retrieve,
        )
        self.update = to_raw_response_wrapper(
            profile.update,
        )

    @cached_property
    def photo(self) -> PhotoResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return PhotoResourceWithRawResponse(self._profile.photo)


class AsyncProfileResourceWithRawResponse:
    def __init__(self, profile: AsyncProfileResource) -> None:
        self._profile = profile

        self.retrieve = async_to_raw_response_wrapper(
            profile.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            profile.update,
        )

    @cached_property
    def photo(self) -> AsyncPhotoResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncPhotoResourceWithRawResponse(self._profile.photo)


class ProfileResourceWithStreamingResponse:
    def __init__(self, profile: ProfileResource) -> None:
        self._profile = profile

        self.retrieve = to_streamed_response_wrapper(
            profile.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            profile.update,
        )

    @cached_property
    def photo(self) -> PhotoResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return PhotoResourceWithStreamingResponse(self._profile.photo)


class AsyncProfileResourceWithStreamingResponse:
    def __init__(self, profile: AsyncProfileResource) -> None:
        self._profile = profile

        self.retrieve = async_to_streamed_response_wrapper(
            profile.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            profile.update,
        )

    @cached_property
    def photo(self) -> AsyncPhotoResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncPhotoResourceWithStreamingResponse(self._profile.photo)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..types import global_ip_health_check_list_params, global_ip_health_check_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.global_ip_health_check_list_response import GlobalIPHealthCheckListResponse
from ..types.global_ip_health_check_create_response import GlobalIPHealthCheckCreateResponse
from ..types.global_ip_health_check_delete_response import GlobalIPHealthCheckDeleteResponse
from ..types.global_ip_health_check_retrieve_response import GlobalIPHealthCheckRetrieveResponse

__all__ = ["GlobalIPHealthChecksResource", "AsyncGlobalIPHealthChecksResource"]


class GlobalIPHealthChecksResource(SyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> GlobalIPHealthChecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return GlobalIPHealthChecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalIPHealthChecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return GlobalIPHealthChecksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        global_ip_id: str | Omit = omit,
        health_check_params: Dict[str, object] | Omit = omit,
        health_check_type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckCreateResponse:
        """
        Create a Global IP health check.

        Args:
          global_ip_id: Global IP ID.

          health_check_params: A Global IP health check params.

          health_check_type: The Global IP health check type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/global_ip_health_checks",
            body=maybe_transform(
                {
                    "global_ip_id": global_ip_id,
                    "health_check_params": health_check_params,
                    "health_check_type": health_check_type,
                },
                global_ip_health_check_create_params.GlobalIPHealthCheckCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckRetrieveResponse:
        """
        Retrieve a Global IP health check.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/global_ip_health_checks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[GlobalIPHealthCheckListResponse]:
        """
        List all Global IP health checks.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/global_ip_health_checks",
            page=SyncDefaultFlatPagination[GlobalIPHealthCheckListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    global_ip_health_check_list_params.GlobalIPHealthCheckListParams,
                ),
            ),
            model=GlobalIPHealthCheckListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckDeleteResponse:
        """
        Delete a Global IP health check.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/global_ip_health_checks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckDeleteResponse,
        )


class AsyncGlobalIPHealthChecksResource(AsyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> AsyncGlobalIPHealthChecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalIPHealthChecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalIPHealthChecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncGlobalIPHealthChecksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        global_ip_id: str | Omit = omit,
        health_check_params: Dict[str, object] | Omit = omit,
        health_check_type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckCreateResponse:
        """
        Create a Global IP health check.

        Args:
          global_ip_id: Global IP ID.

          health_check_params: A Global IP health check params.

          health_check_type: The Global IP health check type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/global_ip_health_checks",
            body=await async_maybe_transform(
                {
                    "global_ip_id": global_ip_id,
                    "health_check_params": health_check_params,
                    "health_check_type": health_check_type,
                },
                global_ip_health_check_create_params.GlobalIPHealthCheckCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckRetrieveResponse:
        """
        Retrieve a Global IP health check.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/global_ip_health_checks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[GlobalIPHealthCheckListResponse, AsyncDefaultFlatPagination[GlobalIPHealthCheckListResponse]]:
        """
        List all Global IP health checks.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/global_ip_health_checks",
            page=AsyncDefaultFlatPagination[GlobalIPHealthCheckListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    global_ip_health_check_list_params.GlobalIPHealthCheckListParams,
                ),
            ),
            model=GlobalIPHealthCheckListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPHealthCheckDeleteResponse:
        """
        Delete a Global IP health check.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/global_ip_health_checks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPHealthCheckDeleteResponse,
        )


class GlobalIPHealthChecksResourceWithRawResponse:
    def __init__(self, global_ip_health_checks: GlobalIPHealthChecksResource) -> None:
        self._global_ip_health_checks = global_ip_health_checks

        self.create = to_raw_response_wrapper(
            global_ip_health_checks.create,
        )
        self.retrieve = to_raw_response_wrapper(
            global_ip_health_checks.retrieve,
        )
        self.list = to_raw_response_wrapper(
            global_ip_health_checks.list,
        )
        self.delete = to_raw_response_wrapper(
            global_ip_health_checks.delete,
        )


class AsyncGlobalIPHealthChecksResourceWithRawResponse:
    def __init__(self, global_ip_health_checks: AsyncGlobalIPHealthChecksResource) -> None:
        self._global_ip_health_checks = global_ip_health_checks

        self.create = async_to_raw_response_wrapper(
            global_ip_health_checks.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            global_ip_health_checks.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            global_ip_health_checks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            global_ip_health_checks.delete,
        )


class GlobalIPHealthChecksResourceWithStreamingResponse:
    def __init__(self, global_ip_health_checks: GlobalIPHealthChecksResource) -> None:
        self._global_ip_health_checks = global_ip_health_checks

        self.create = to_streamed_response_wrapper(
            global_ip_health_checks.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            global_ip_health_checks.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            global_ip_health_checks.list,
        )
        self.delete = to_streamed_response_wrapper(
            global_ip_health_checks.delete,
        )


class AsyncGlobalIPHealthChecksResourceWithStreamingResponse:
    def __init__(self, global_ip_health_checks: AsyncGlobalIPHealthChecksResource) -> None:
        self._global_ip_health_checks = global_ip_health_checks

        self.create = async_to_streamed_response_wrapper(
            global_ip_health_checks.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            global_ip_health_checks.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            global_ip_health_checks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            global_ip_health_checks.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..types import (
    voice_clone_list_params,
    voice_clone_create_params,
    voice_clone_update_params,
    voice_clone_create_from_upload_params,
)
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, FileTypes, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    to_custom_raw_response_wrapper,
    async_to_streamed_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.voice_clone_data import VoiceCloneData
from ..types.voice_clone_create_response import VoiceCloneCreateResponse
from ..types.voice_clone_update_response import VoiceCloneUpdateResponse
from ..types.voice_clone_create_from_upload_response import VoiceCloneCreateFromUploadResponse

__all__ = ["VoiceClonesResource", "AsyncVoiceClonesResource"]


class VoiceClonesResource(SyncAPIResource):
    """
    Capture and manage voice identities as clones for use in text-to-speech synthesis.
    """

    @cached_property
    def with_raw_response(self) -> VoiceClonesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoiceClonesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceClonesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoiceClonesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        gender: Literal["male", "female", "neutral"],
        language: str,
        name: str,
        voice_design_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneCreateResponse:
        """
        Creates a new voice clone by capturing the voice identity of an existing voice
        design. The clone can then be used for text-to-speech synthesis.

        Args:
          gender: Gender of the voice clone.

          language: ISO 639-1 language code for the clone (e.g. `en`, `fr`, `de`).

          name: Name for the voice clone.

          voice_design_id: UUID of the source voice design to clone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/voice_clones",
            body=maybe_transform(
                {
                    "gender": gender,
                    "language": language,
                    "name": name,
                    "voice_design_id": voice_design_id,
                },
                voice_clone_create_params.VoiceCloneCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneCreateResponse,
        )

    def update(
        self,
        id: str,
        *,
        name: str,
        gender: Literal["male", "female", "neutral"] | Omit = omit,
        language: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneUpdateResponse:
        """
        Updates the name, language, or gender of a voice clone.

        Args:
          name: New name for the voice clone.

          gender: Updated gender for the voice clone.

          language: Updated ISO 639-1 language code or `auto`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/voice_clones/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "gender": gender,
                    "language": language,
                },
                voice_clone_update_params.VoiceCloneUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneUpdateResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["name", "-name", "created_at", "-created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[VoiceCloneData]:
        """
        Returns a paginated list of voice clones belonging to the authenticated account.

        Args:
          filter_name: Case-insensitive substring filter on the name field.

          page_number: Page number for pagination (1-based).

          page_size: Number of results per page.

          sort: Sort order. Prefix with `-` for descending. Defaults to `-created_at`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/voice_clones",
            page=SyncDefaultFlatPagination[VoiceCloneData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_clone_list_params.VoiceCloneListParams,
                ),
            ),
            model=VoiceCloneData,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a voice clone.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/voice_clones/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def create_from_upload(
        self,
        *,
        audio_file: FileTypes,
        language: str,
        name: str,
        gender: Literal["male", "female", "neutral"] | Omit = omit,
        label: str | Omit = omit,
        ref_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneCreateFromUploadResponse:
        """Creates a new voice clone by uploading an audio file directly.

        Supported
        formats: WAV, MP3, FLAC, OGG, M4A. For best results, provide 5–10 seconds of
        clear speech. Maximum file size: 2MB.

        Args:
          audio_file: Audio file to clone the voice from. Supported formats: WAV, MP3, FLAC, OGG, M4A.
              For best quality, provide 5–10 seconds of clear, uninterrupted speech. Maximum
              size: 2MB.

          language: ISO 639-1 language code (e.g. `en`, `fr`) or `auto` for automatic detection.

          name: Name for the voice clone.

          gender: Gender of the voice clone.

          label: Optional custom label describing the voice style. If omitted, falls back to the
              source design's prompt text.

          ref_text: Optional transcript of the audio file. Providing this improves clone quality.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "audio_file": audio_file,
                "language": language,
                "name": name,
                "gender": gender,
                "label": label,
                "ref_text": ref_text,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["audio_file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/voice_clones/from_upload",
            body=maybe_transform(body, voice_clone_create_from_upload_params.VoiceCloneCreateFromUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneCreateFromUploadResponse,
        )

    def download_sample(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Downloads the WAV audio sample that was used to create the voice clone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "audio/wav", **(extra_headers or {})}
        return self._get(
            f"/voice_clones/{id}/sample",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncVoiceClonesResource(AsyncAPIResource):
    """
    Capture and manage voice identities as clones for use in text-to-speech synthesis.
    """

    @cached_property
    def with_raw_response(self) -> AsyncVoiceClonesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceClonesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceClonesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoiceClonesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        gender: Literal["male", "female", "neutral"],
        language: str,
        name: str,
        voice_design_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneCreateResponse:
        """
        Creates a new voice clone by capturing the voice identity of an existing voice
        design. The clone can then be used for text-to-speech synthesis.

        Args:
          gender: Gender of the voice clone.

          language: ISO 639-1 language code for the clone (e.g. `en`, `fr`, `de`).

          name: Name for the voice clone.

          voice_design_id: UUID of the source voice design to clone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/voice_clones",
            body=await async_maybe_transform(
                {
                    "gender": gender,
                    "language": language,
                    "name": name,
                    "voice_design_id": voice_design_id,
                },
                voice_clone_create_params.VoiceCloneCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneCreateResponse,
        )

    async def update(
        self,
        id: str,
        *,
        name: str,
        gender: Literal["male", "female", "neutral"] | Omit = omit,
        language: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneUpdateResponse:
        """
        Updates the name, language, or gender of a voice clone.

        Args:
          name: New name for the voice clone.

          gender: Updated gender for the voice clone.

          language: Updated ISO 639-1 language code or `auto`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/voice_clones/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "gender": gender,
                    "language": language,
                },
                voice_clone_update_params.VoiceCloneUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneUpdateResponse,
        )

    def list(
        self,
        *,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["name", "-name", "created_at", "-created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VoiceCloneData, AsyncDefaultFlatPagination[VoiceCloneData]]:
        """
        Returns a paginated list of voice clones belonging to the authenticated account.

        Args:
          filter_name: Case-insensitive substring filter on the name field.

          page_number: Page number for pagination (1-based).

          page_size: Number of results per page.

          sort: Sort order. Prefix with `-` for descending. Defaults to `-created_at`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/voice_clones",
            page=AsyncDefaultFlatPagination[VoiceCloneData],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    voice_clone_list_params.VoiceCloneListParams,
                ),
            ),
            model=VoiceCloneData,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently deletes a voice clone.

        This action cannot be undone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/voice_clones/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def create_from_upload(
        self,
        *,
        audio_file: FileTypes,
        language: str,
        name: str,
        gender: Literal["male", "female", "neutral"] | Omit = omit,
        label: str | Omit = omit,
        ref_text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoiceCloneCreateFromUploadResponse:
        """Creates a new voice clone by uploading an audio file directly.

        Supported
        formats: WAV, MP3, FLAC, OGG, M4A. For best results, provide 5–10 seconds of
        clear speech. Maximum file size: 2MB.

        Args:
          audio_file: Audio file to clone the voice from. Supported formats: WAV, MP3, FLAC, OGG, M4A.
              For best quality, provide 5–10 seconds of clear, uninterrupted speech. Maximum
              size: 2MB.

          language: ISO 639-1 language code (e.g. `en`, `fr`) or `auto` for automatic detection.

          name: Name for the voice clone.

          gender: Gender of the voice clone.

          label: Optional custom label describing the voice style. If omitted, falls back to the
              source design's prompt text.

          ref_text: Optional transcript of the audio file. Providing this improves clone quality.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "audio_file": audio_file,
                "language": language,
                "name": name,
                "gender": gender,
                "label": label,
                "ref_text": ref_text,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["audio_file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/voice_clones/from_upload",
            body=await async_maybe_transform(
                body, voice_clone_create_from_upload_params.VoiceCloneCreateFromUploadParams
            ),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoiceCloneCreateFromUploadResponse,
        )

    async def download_sample(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Downloads the WAV audio sample that was used to create the voice clone.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "audio/wav", **(extra_headers or {})}
        return await self._get(
            f"/voice_clones/{id}/sample",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class VoiceClonesResourceWithRawResponse:
    def __init__(self, voice_clones: VoiceClonesResource) -> None:
        self._voice_clones = voice_clones

        self.create = to_raw_response_wrapper(
            voice_clones.create,
        )
        self.update = to_raw_response_wrapper(
            voice_clones.update,
        )
        self.list = to_raw_response_wrapper(
            voice_clones.list,
        )
        self.delete = to_raw_response_wrapper(
            voice_clones.delete,
        )
        self.create_from_upload = to_raw_response_wrapper(
            voice_clones.create_from_upload,
        )
        self.download_sample = to_custom_raw_response_wrapper(
            voice_clones.download_sample,
            BinaryAPIResponse,
        )


class AsyncVoiceClonesResourceWithRawResponse:
    def __init__(self, voice_clones: AsyncVoiceClonesResource) -> None:
        self._voice_clones = voice_clones

        self.create = async_to_raw_response_wrapper(
            voice_clones.create,
        )
        self.update = async_to_raw_response_wrapper(
            voice_clones.update,
        )
        self.list = async_to_raw_response_wrapper(
            voice_clones.list,
        )
        self.delete = async_to_raw_response_wrapper(
            voice_clones.delete,
        )
        self.create_from_upload = async_to_raw_response_wrapper(
            voice_clones.create_from_upload,
        )
        self.download_sample = async_to_custom_raw_response_wrapper(
            voice_clones.download_sample,
            AsyncBinaryAPIResponse,
        )


class VoiceClonesResourceWithStreamingResponse:
    def __init__(self, voice_clones: VoiceClonesResource) -> None:
        self._voice_clones = voice_clones

        self.create = to_streamed_response_wrapper(
            voice_clones.create,
        )
        self.update = to_streamed_response_wrapper(
            voice_clones.update,
        )
        self.list = to_streamed_response_wrapper(
            voice_clones.list,
        )
        self.delete = to_streamed_response_wrapper(
            voice_clones.delete,
        )
        self.create_from_upload = to_streamed_response_wrapper(
            voice_clones.create_from_upload,
        )
        self.download_sample = to_custom_streamed_response_wrapper(
            voice_clones.download_sample,
            StreamedBinaryAPIResponse,
        )


class AsyncVoiceClonesResourceWithStreamingResponse:
    def __init__(self, voice_clones: AsyncVoiceClonesResource) -> None:
        self._voice_clones = voice_clones

        self.create = async_to_streamed_response_wrapper(
            voice_clones.create,
        )
        self.update = async_to_streamed_response_wrapper(
            voice_clones.update,
        )
        self.list = async_to_streamed_response_wrapper(
            voice_clones.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            voice_clones.delete,
        )
        self.create_from_upload = async_to_streamed_response_wrapper(
            voice_clones.create_from_upload,
        )
        self.download_sample = async_to_custom_streamed_response_wrapper(
            voice_clones.download_sample,
            AsyncStreamedBinaryAPIResponse,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["OAuthIntrospectResponse"]


class OAuthIntrospectResponse(BaseModel):
    active: bool
    """Whether the token is active"""

    aud: Optional[str] = None
    """Audience"""

    client_id: Optional[str] = None
    """Client identifier"""

    exp: Optional[int] = None
    """Expiration timestamp"""

    iat: Optional[int] = None
    """Issued at timestamp"""

    iss: Optional[str] = None
    """Issuer"""

    scope: Optional[str] = None
    """Space-separated list of scopes"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from .tf_verification_status import TfVerificationStatus

__all__ = ["RequestRetrieveStatusHistoryResponse", "Record"]


class Record(BaseModel):
    """A single entry in the verification request status history"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """The timestamp at which this status change occurred"""

    verification_status: TfVerificationStatus = FieldInfo(alias="verificationStatus")
    """Tollfree verification status"""

    reason: Optional[str] = None
    """An explanation of why this request has its current status."""


class RequestRetrieveStatusHistoryResponse(BaseModel):
    """A paginated response"""

    records: List[Record]
    """The records yielded by this request"""

    total_records: int
    """The total amount of records for these query parameters"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ActionGenerateJoinClientTokenParams"]


class ActionGenerateJoinClientTokenParams(TypedDict, total=False):
    refresh_token_ttl_secs: int
    """
    The time to live in seconds of the Refresh Token, after that time the Refresh
    Token is invalid and can't be used to refresh Client Token.
    """

    token_ttl_secs: int
    """
    The time to live in seconds of the Client Token, after that time the Client
    Token is invalid and can't be used to join a Room.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .country_coverage import CountryCoverage

__all__ = ["CountryCoverageRetrieveCountryResponse"]


class CountryCoverageRetrieveCountryResponse(BaseModel):
    data: Optional[CountryCoverage] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .stream_codec import StreamCodec
from .sip_header_param import SipHeaderParam
from .custom_sip_header_param import CustomSipHeaderParam
from .dialogflow_config_param import DialogflowConfigParam
from .sound_modifications_param import SoundModificationsParam
from .stream_bidirectional_mode import StreamBidirectionalMode
from .stream_bidirectional_codec import StreamBidirectionalCodec
from .stream_bidirectional_target_legs import StreamBidirectionalTargetLegs
from .stream_bidirectional_sampling_rate import StreamBidirectionalSamplingRate
from .calls.transcription_start_request_param import TranscriptionStartRequestParam

__all__ = ["CallDialParams", "AnsweringMachineDetectionConfig", "ConferenceConfig"]


class CallDialParams(TypedDict, total=False):
    connection_id: Required[str]
    """
    The ID of the Call Control App (formerly ID of the connection) to be used when
    dialing the destination.
    """

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """
    The `from` number to be used as the caller id presented to the destination (`to`
    number). The number should be in +E164 format.
    """

    to: Required[Union[str, SequenceNotStr[str]]]
    """The DID or SIP URI to dial out to.

    Multiple DID or SIP URIs can be provided using an array of strings
    """

    answering_machine_detection: Literal["premium", "detect", "detect_beep", "detect_words", "greeting_end", "disabled"]
    """Enables Answering Machine Detection.

    Telnyx offers Premium and Standard detections. With Premium detection, when a
    call is answered, Telnyx runs real-time detection and sends a
    `call.machine.premium.detection.ended` webhook with one of the following
    results: `human_residence`, `human_business`, `machine`, `silence` or
    `fax_detected`. If we detect a beep, we also send a
    `call.machine.premium.greeting.ended` webhook with the result of
    `beep_detected`. If we detect a beep before
    `call.machine.premium.detection.ended` we only send
    `call.machine.premium.greeting.ended`, and if we detect a beep after
    `call.machine.premium.detection.ended`, we send both webhooks. With Standard
    detection, when a call is answered, Telnyx runs real-time detection to determine
    if it was picked up by a human or a machine and sends an
    `call.machine.detection.ended` webhook with the analysis result. If
    `greeting_end` or `detect_words` is used and a `machine` is detected, you will
    receive another `call.machine.greeting.ended` webhook when the answering machine
    greeting ends with a beep or silence. If `detect_beep` is used, you will only
    receive `call.machine.greeting.ended` if a beep is detected.
    """

    answering_machine_detection_config: AnsweringMachineDetectionConfig
    """
    Optional configuration parameters to modify 'answering_machine_detection'
    performance.
    """

    audio_url: str
    """The URL of a file to be played back to the callee when the call is answered.

    The URL can point to either a WAV or MP3 file. media_name and audio_url cannot
    be used together in one request.
    """

    billing_group_id: str
    """Use this field to set the Billing Group ID for the call.

    Must be a valid and existing Billing Group ID.
    """

    bridge_intent: bool
    """Indicates the intent to bridge this call with the call specified in link_to.

    When bridge_intent is true, link_to becomes required and the from number will be
    overwritten by the from number from the linked call.
    """

    bridge_on_answer: bool
    """Whether to automatically bridge answered call to the call specified in link_to.

    When bridge_on_answer is true, link_to becomes required.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore others Dial commands with the same `command_id`.
    """

    conference_config: ConferenceConfig
    """Optional configuration parameters to dial new participant into a conference."""

    custom_headers: Iterable[CustomSipHeaderParam]
    """Custom headers to be added to the SIP INVITE."""

    dialogflow_config: DialogflowConfigParam

    enable_dialogflow: bool
    """Enables Dialogflow for the current call. The default value is false."""

    from_display_name: str
    """
    The `from_display_name` string to be used as the caller id name (SIP From
    Display Name) presented to the destination (`to` number). The string should have
    a maximum of 128 characters, containing only letters, numbers, spaces, and
    -\\__~!.+ special characters. If ommited, the display name will be the same as the
    number in the `from` field.
    """

    link_to: str
    """Use another call's control id for sharing the same call session id"""

    media_encryption: Literal["disabled", "SRTP", "DTLS"]
    """Defines whether media should be encrypted on the call."""

    media_name: str
    """
    The media_name of a file to be played back to the callee when the call is
    answered. The media_name must point to a file previously uploaded to
    api.telnyx.com/v2/media by the same user/organization. The file must either be a
    WAV or MP3 file.
    """

    park_after_unbridge: str
    """If supplied with the value `self`, the current leg will be parked after
    unbridge.

    If not set, the default behavior is to hang up the leg. When park_after_unbridge
    is set, link_to becomes required.
    """

    preferred_codecs: str
    """
    The list of comma-separated codecs in a preferred order for the forked media to
    be received.
    """

    prevent_double_bridge: bool
    """Prevents bridging and hangs up the call if the target is already bridged.

    Disabled by default.
    """

    record: Literal["record-from-answer"]
    """Start recording automatically after an event. Disabled by default."""

    record_channels: Literal["single", "dual"]
    """
    Defines which channel should be recorded ('single' or 'dual') when `record` is
    specified.
    """

    record_custom_file_name: str
    """The custom recording file name to be used instead of the default `call_leg_id`.

    Telnyx will still add a Unix timestamp suffix.
    """

    record_format: Literal["wav", "mp3"]
    """
    Defines the format of the recording ('wav' or 'mp3') when `record` is specified.
    """

    record_max_length: int
    """
    Defines the maximum length for the recording in seconds when `record` is
    specified. The minimum value is 0. The maximum value is 43200. The default value
    is 0 (infinite).
    """

    record_timeout_secs: int
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected when `record` is specified. The timer only starts when the
    speech is detected. Please note that call transcription is used to detect
    silence and the related charge will be applied. The minimum value is 0. The
    default value is 0 (infinite).
    """

    record_track: Literal["both", "inbound", "outbound"]
    """The audio track to be recorded.

    Can be either `both`, `inbound` or `outbound`. If only single track is specified
    (`inbound`, `outbound`), `channels` configuration is ignored and it will be
    recorded as mono (single channel).
    """

    record_trim: Literal["trim-silence"]
    """
    When set to `trim-silence`, silence will be removed from the beginning and end
    of the recording.
    """

    send_silence_when_idle: bool
    """Generate silence RTP packets when no transmission available."""

    sip_auth_password: str
    """SIP Authentication password used for SIP challenges."""

    sip_auth_username: str
    """SIP Authentication username used for SIP challenges."""

    sip_headers: Iterable[SipHeaderParam]
    """SIP headers to be added to the SIP INVITE request.

    Currently only User-to-User header is supported.
    """

    sip_region: Literal["US", "Europe", "Canada", "Australia", "Middle East"]
    """Defines the SIP region to be used for the call."""

    sip_transport_protocol: Literal["UDP", "TCP", "TLS"]
    """Defines SIP transport protocol to be used on the call."""

    sound_modifications: SoundModificationsParam
    """Use this field to modify sound effects, for example adjust the pitch."""

    stream_auth_token: str
    """
    An authentication token to be sent as part of the WebSocket connection when
    using streaming. Maximum length is 4000 characters.
    """

    stream_bidirectional_codec: StreamBidirectionalCodec
    """Indicates codec for bidirectional streaming RTP payloads.

    Used only with stream_bidirectional_mode=rtp. Case sensitive.
    """

    stream_bidirectional_mode: StreamBidirectionalMode
    """Configures method of bidirectional streaming (mp3, rtp)."""

    stream_bidirectional_sampling_rate: StreamBidirectionalSamplingRate
    """Audio sampling rate."""

    stream_bidirectional_target_legs: StreamBidirectionalTargetLegs
    """Specifies which call legs should receive the bidirectional stream audio."""

    stream_codec: StreamCodec
    """Specifies the codec to be used for the streamed audio.

    When set to 'default' or when transcoding is not possible, the codec from the
    call will be used.
    """

    stream_establish_before_call_originate: bool
    """Establish websocket connection before dialing the destination.

    This is useful for cases where the websocket connection takes a long time to
    establish.
    """

    stream_track: Literal["inbound_track", "outbound_track", "both_tracks"]
    """Specifies which track should be streamed."""

    stream_url: str
    """The destination WebSocket address where the stream is going to be delivered."""

    supervise_call_control_id: str
    """The call leg which will be supervised by the new call."""

    supervisor_role: Literal["barge", "whisper", "monitor"]
    """The role of the supervisor call.

    'barge' means that supervisor call hears and is being heard by both ends of the
    call (caller & callee). 'whisper' means that only supervised_call_control_id
    hears supervisor but supervisor can hear everything. 'monitor' means that nobody
    can hear supervisor call, but supervisor can hear everything on the call.
    """

    time_limit_secs: int
    """Sets the maximum duration of a Call Control Leg in seconds.

    If the time limit is reached, the call will hangup and a `call.hangup` webhook
    with a `hangup_cause` of `time_limit` will be sent. For example, by setting a
    time limit of 120 seconds, a Call Leg will be automatically terminated two
    minutes after being answered. The default time limit is 14400 seconds or 4 hours
    and this is also the maximum allowed call length.
    """

    timeout_secs: int
    """
    The number of seconds that Telnyx will wait for the call to be answered by the
    destination to which it is being called. If the timeout is reached before an
    answer is received, the call will hangup and a `call.hangup` webhook with a
    `hangup_cause` of `timeout` will be sent. Minimum value is 5 seconds. Maximum
    value is 600 seconds.
    """

    transcription: bool
    """Enable transcription upon call answer. The default value is false."""

    transcription_config: TranscriptionStartRequestParam

    webhook_url: str
    """
    Use this field to override the URL for which Telnyx will send subsequent
    webhooks to for this call.
    """

    webhook_url_method: Literal["POST", "GET"]
    """HTTP request type used for `webhook_url`."""


class AnsweringMachineDetectionConfig(TypedDict, total=False):
    """
    Optional configuration parameters to modify 'answering_machine_detection' performance.
    """

    after_greeting_silence_millis: int
    """
    Silence duration threshold after a greeting message or voice for it be
    considered human.
    """

    between_words_silence_millis: int
    """Maximum threshold for silence between words."""

    greeting_duration_millis: int
    """Maximum threshold of a human greeting.

    If greeting longer than this value, considered machine.
    """

    greeting_silence_duration_millis: int
    """If machine already detected, maximum threshold for silence between words.

    If exceeded, the greeting is considered ended.
    """

    greeting_total_analysis_time_millis: int
    """
    If machine already detected, maximum timeout threshold to determine the end of
    the machine greeting.
    """

    initial_silence_millis: int
    """If initial silence duration is greater than this value, consider it a machine."""

    maximum_number_of_words: int
    """If number of detected words is greater than this value, consder it a machine."""

    maximum_word_length_millis: int
    """If a single word lasts longer than this threshold, consider it a machine."""

    silence_threshold: int
    """Minimum noise threshold for any analysis."""

    total_analysis_time_millis: int
    """Maximum timeout threshold for overall detection."""


class ConferenceConfig(TypedDict, total=False):
    """Optional configuration parameters to dial new participant into a conference."""

    id: str
    """Conference ID to be joined"""

    beep_enabled: Literal["always", "never", "on_enter", "on_exit"]
    """
    Whether a beep sound should be played when the participant joins and/or leaves
    the conference. Can be used to override the conference-level setting.
    """

    conference_name: str
    """Conference name to be joined"""

    early_media: bool
    """Controls the moment when dialled call is joined into conference.

    If set to `true` user will be joined as soon as media is available (ringback).
    If `false` user will be joined when call is answered. Defaults to `true`
    """

    end_conference_on_exit: bool
    """
    Whether the conference should end and all remaining participants be hung up
    after the participant leaves the conference. Defaults to "false".
    """

    hold: bool
    """
    Whether the participant should be put on hold immediately after joining the
    conference. Defaults to "false".
    """

    hold_audio_url: str
    """
    The URL of a file to be played to the participant when they are put on hold
    after joining the conference. hold_media_name and hold_audio_url cannot be used
    together in one request. Takes effect only when "start_conference_on_create" is
    set to "false". This property takes effect only if "hold" is set to "true".
    """

    hold_media_name: str
    """
    The media_name of a file to be played to the participant when they are put on
    hold after joining the conference. The media_name must point to a file
    previously uploaded to api.telnyx.com/v2/media by the same user/organization.
    The file must either be a WAV or MP3 file. Takes effect only when
    "start_conference_on_create" is set to "false". This property takes effect only
    if "hold" is set to "true".
    """

    mute: bool
    """Whether the participant should be muted immediately after joining the
    conference.

    Defaults to "false".
    """

    soft_end_conference_on_exit: bool
    """Whether the conference should end after the participant leaves the conference.

    NOTE this doesn't hang up the other participants. Defaults to "false".
    """

    start_conference_on_create: bool
    """Whether the conference should be started on creation.

    If the conference isn't started all participants that join are automatically put
    on hold. Defaults to "true".
    """

    start_conference_on_enter: bool
    """
    Whether the conference should be started after the participant joins the
    conference. Defaults to "false".
    """

    supervisor_role: Literal["barge", "monitor", "none", "whisper"]
    """Sets the joining participant as a supervisor for the conference.

    A conference can have multiple supervisors. "barge" means the supervisor enters
    the conference as a normal participant. This is the same as "none". "monitor"
    means the supervisor is muted but can hear all participants. "whisper" means
    that only the specified "whisper_call_control_ids" can hear the supervisor.
    Defaults to "none".
    """

    whisper_call_control_ids: SequenceNotStr[str]
    """Array of unique call_control_ids the joining supervisor can whisper to.

    If none provided, the supervisor will join the conference as a monitoring
    participant only.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["JsonRetrieveRecordingTranscriptionSidJsonResponse"]


class JsonRetrieveRecordingTranscriptionSidJsonResponse(BaseModel):
    account_sid: Optional[str] = None

    api_version: Optional[str] = None
    """The version of the API that was used to make the request."""

    call_sid: Optional[str] = None

    date_created: Optional[datetime] = None

    date_updated: Optional[datetime] = None

    duration: Optional[str] = None
    """The duration of this recording, given in seconds."""

    recording_sid: Optional[str] = None
    """Identifier of a resource."""

    sid: Optional[str] = None
    """Identifier of a resource."""

    status: Optional[Literal["in-progress", "completed"]] = None
    """The status of the recording transcriptions.

    The transcription text will be available only when the status is completed.
    """

    transcription_text: Optional[str] = None
    """The recording's transcribed text"""

    uri: Optional[str] = None
    """The relative URI for the recording transcription resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["ShortCodeUpdateParams"]


class ShortCodeUpdateParams(TypedDict, total=False):
    messaging_profile_id: Required[str]
    """Unique identifier for a messaging profile."""

    tags: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["RecursiveCluster", "Node"]


class Node(BaseModel):
    filename: str
    """
    The corresponding source file of your embedded storage bucket that the node is
    from.
    """

    text: str
    """The text of the node."""


class RecursiveCluster(BaseModel):
    cluster_id: str

    cluster_summary: str

    total_number_of_nodes: int

    cluster_header: Optional[str] = None

    nodes: Optional[List[Node]] = None

    subclusters: Optional[List["RecursiveCluster"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_playback_started import CallPlaybackStarted

__all__ = ["CallPlaybackStartedWebhookEvent"]


class CallPlaybackStartedWebhookEvent(BaseModel):
    data: Optional[CallPlaybackStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .telnyx_conversation_channel import TelnyxConversationChannel

__all__ = ["TestUpdateParams", "Rubric"]


class TestUpdateParams(TypedDict, total=False):
    description: str
    """Updated description of the test's purpose and evaluation criteria."""

    destination: str
    """Updated target destination for test conversations."""

    instructions: str
    """Updated test scenario instructions and objectives."""

    max_duration_seconds: int
    """Updated maximum test duration in seconds."""

    name: str
    """Updated name for the assistant test. Must be unique and descriptive."""

    rubric: Iterable[Rubric]
    """Updated evaluation criteria for assessing assistant performance."""

    telnyx_conversation_channel: TelnyxConversationChannel
    """Updated communication channel for the test execution."""

    test_suite: str
    """Updated test suite assignment for better organization."""


class Rubric(TypedDict, total=False):
    criteria: Required[str]
    """
    Specific guidance on how to assess the assistant’s performance for this rubric
    item.
    """

    name: Required[str]
    """Label for the evaluation criterion, e.g., Empathy, Accuracy, Clarity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["JobDeletePhoneNumberBlockParams"]


class JobDeletePhoneNumberBlockParams(TypedDict, total=False):
    phone_number_block_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .user_requirement import UserRequirement

__all__ = ["RequirementGroup"]


class RequirementGroup(BaseModel):
    id: Optional[str] = None

    action: Optional[str] = None

    country_code: Optional[str] = None

    created_at: Optional[datetime] = None

    customer_reference: Optional[str] = None

    phone_number_type: Optional[str] = None

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[UserRequirement]] = None

    status: Optional[Literal["approved", "unapproved", "pending-approval", "declined", "expired"]] = None

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RimeVoiceSettings"]


class RimeVoiceSettings(BaseModel):
    type: Literal["rime"]
    """Voice settings provider type"""

    voice_speed: Optional[float] = None
    """Speech speed multiplier. Default is 1.0."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VoiceCloneData"]


class VoiceCloneData(BaseModel):
    """A voice clone object."""

    id: Optional[str] = None
    """Unique identifier for the voice clone."""

    created_at: Optional[datetime] = None
    """Timestamp when the voice clone was created."""

    gender: Optional[Literal["male", "female", "neutral"]] = None
    """Gender of the voice clone."""

    label: Optional[str] = None
    """Voice style description.

    If not explicitly set on upload, falls back to the source design's prompt text.
    """

    language: Optional[str] = None
    """ISO 639-1 language code of the voice clone."""

    name: Optional[str] = None
    """Name of the voice clone."""

    record_type: Optional[Literal["voice_clone"]] = None
    """Identifies the resource type."""

    source_voice_design_id: Optional[str] = None
    """UUID of the source voice design. `null` for upload-based clones."""

    source_voice_design_version: Optional[int] = None
    """Version of the source voice design used. `null` for upload-based clones."""

    updated_at: Optional[datetime] = None
    """Timestamp when the voice clone was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["UsageGetAPIUsageParams", "Filter"]


class UsageGetAPIUsageParams(TypedDict, total=False):
    filter: Required[Filter]
    """Consolidated filter parameter (deepObject style).

    Originally: filter[start_time], filter[end_time]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[start_time], filter[end_time]
    """

    end_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The end time of the period to filter the usage (ISO microsecond format)"""

    start_time: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The start time of the period to filter the usage (ISO microsecond format)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .record import Record

__all__ = ["NetworkListResponse"]


class NetworkListResponse(Record):
    name: Optional[str] = None
    """A user specified name for the network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .runs import (
    RunsResource,
    AsyncRunsResource,
    RunsResourceWithRawResponse,
    AsyncRunsResourceWithRawResponse,
    RunsResourceWithStreamingResponse,
    AsyncRunsResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .missions import (
    MissionsResource,
    AsyncMissionsResource,
    MissionsResourceWithRawResponse,
    AsyncMissionsResourceWithRawResponse,
    MissionsResourceWithStreamingResponse,
    AsyncMissionsResourceWithStreamingResponse,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)

__all__ = [
    "RunsResource",
    "AsyncRunsResource",
    "RunsResourceWithRawResponse",
    "AsyncRunsResourceWithRawResponse",
    "RunsResourceWithStreamingResponse",
    "AsyncRunsResourceWithStreamingResponse",
    "KnowledgeBasesResource",
    "AsyncKnowledgeBasesResource",
    "KnowledgeBasesResourceWithRawResponse",
    "AsyncKnowledgeBasesResourceWithRawResponse",
    "KnowledgeBasesResourceWithStreamingResponse",
    "AsyncKnowledgeBasesResourceWithStreamingResponse",
    "McpServersResource",
    "AsyncMcpServersResource",
    "McpServersResourceWithRawResponse",
    "AsyncMcpServersResourceWithRawResponse",
    "McpServersResourceWithStreamingResponse",
    "AsyncMcpServersResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "MissionsResource",
    "AsyncMissionsResource",
    "MissionsResourceWithRawResponse",
    "AsyncMissionsResourceWithRawResponse",
    "MissionsResourceWithStreamingResponse",
    "AsyncMissionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TemplateListParams"]


class TemplateListParams(TypedDict, total=False):
    filter_category: Annotated[
        Literal["MARKETING", "UTILITY", "AUTHENTICATION"], PropertyInfo(alias="filter[category]")
    ]
    """Filter by category"""

    filter_search: Annotated[str, PropertyInfo(alias="filter[search]")]
    """Search templates by name"""

    filter_status: Annotated[str, PropertyInfo(alias="filter[status]")]
    """Filter by template status"""

    filter_waba_id: Annotated[str, PropertyInfo(alias="filter[waba_id]")]
    """Filter by WABA ID"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["VoiceDesignRenameResponse", "Data"]


class Data(BaseModel):
    """A summarized voice design object (without version-specific fields)."""

    id: Optional[str] = None
    """Unique identifier for the voice design."""

    created_at: Optional[datetime] = None
    """Timestamp when the voice design was first created."""

    name: Optional[str] = None
    """Name of the voice design."""

    record_type: Optional[Literal["voice_design"]] = None
    """Identifies the resource type."""

    updated_at: Optional[datetime] = None
    """Timestamp when the voice design was last updated."""


class VoiceDesignRenameResponse(BaseModel):
    """
    Response envelope for a voice design after a rename operation (no version-specific fields).
    """

    data: Optional[Data] = None
    """A summarized voice design object (without version-specific fields)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .bulk_sim_card_action import BulkSimCardAction

__all__ = ["ActionBulkDisableVoiceResponse"]


class ActionBulkDisableVoiceResponse(BaseModel):
    data: Optional[BulkSimCardAction] = None
    """This object represents a bulk SIM card action.

    It groups SIM card actions created through a bulk endpoint under a single
    resource for further lookup.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .usecase_get_cost_params import UsecaseGetCostParams as UsecaseGetCostParams
from .usecase_get_cost_response import UsecaseGetCostResponse as UsecaseGetCostResponse
from .osr_get_attributes_response import OsrGetAttributesResponse as OsrGetAttributesResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["MetadataFieldMapping"]


class MetadataFieldMapping(BaseModel):
    local_field: str

    parent_field: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import usage_report_list_params, usage_report_get_options_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, strip_not_given, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.usage_report_list_response import UsageReportListResponse
from ..types.usage_report_get_options_response import UsageReportGetOptionsResponse

__all__ = ["UsageReportsResource", "AsyncUsageReportsResource"]


class UsageReportsResource(SyncAPIResource):
    """Usage data reporting across Telnyx products"""

    @cached_property
    def with_raw_response(self) -> UsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UsageReportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        dimensions: SequenceNotStr[str],
        metrics: SequenceNotStr[str],
        product: str,
        date_range: str | Omit = omit,
        end_date: str | Omit = omit,
        filter: str | Omit = omit,
        format: Literal["csv", "json"] | Omit = omit,
        managed_accounts: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: SequenceNotStr[str] | Omit = omit,
        start_date: str | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[UsageReportListResponse]:
        """
        Get Telnyx usage data by product, broken out by the specified dimensions

        Args:
          dimensions: Breakout by specified product dimensions

          metrics: Specified product usage values

          product: Telnyx product

          date_range: A more user-friendly way to specify the timespan you want to filter by. More
              options can be found in the Telnyx API Reference docs.

          end_date: The end date for the time range you are interested in. The maximum time range is
              31 days. Format: YYYY-MM-DDTHH:mm:ssZ

          filter: Filter records on dimensions

          format: Specify the response format (csv or json). JSON is returned by default, even if
              not specified.

          managed_accounts: Return the aggregations for all Managed Accounts under the user making the
              request.

          sort: Specifies the sort order for results

          start_date: The start date for the time range you are interested in. The maximum time range
              is 31 days. Format: YYYY-MM-DDTHH:mm:ssZ

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/usage_reports",
            page=SyncDefaultFlatPagination[UsageReportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dimensions": dimensions,
                        "metrics": metrics,
                        "product": product,
                        "date_range": date_range,
                        "end_date": end_date,
                        "filter": filter,
                        "format": format,
                        "managed_accounts": managed_accounts,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                        "start_date": start_date,
                    },
                    usage_report_list_params.UsageReportListParams,
                ),
            ),
            model=UsageReportListResponse,
        )

    def get_options(
        self,
        *,
        product: str | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportGetOptionsResponse:
        """
        Get the Usage Reports options for querying usage, including the products
        available and their respective metrics and dimensions

        Args:
          product: Options (dimensions and metrics) for a given product. If none specified, all
              products will be returned.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get(
            "/usage_reports/options",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"product": product}, usage_report_get_options_params.UsageReportGetOptionsParams
                ),
            ),
            cast_to=UsageReportGetOptionsResponse,
        )


class AsyncUsageReportsResource(AsyncAPIResource):
    """Usage data reporting across Telnyx products"""

    @cached_property
    def with_raw_response(self) -> AsyncUsageReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsageReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsageReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUsageReportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        dimensions: SequenceNotStr[str],
        metrics: SequenceNotStr[str],
        product: str,
        date_range: str | Omit = omit,
        end_date: str | Omit = omit,
        filter: str | Omit = omit,
        format: Literal["csv", "json"] | Omit = omit,
        managed_accounts: bool | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: SequenceNotStr[str] | Omit = omit,
        start_date: str | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UsageReportListResponse, AsyncDefaultFlatPagination[UsageReportListResponse]]:
        """
        Get Telnyx usage data by product, broken out by the specified dimensions

        Args:
          dimensions: Breakout by specified product dimensions

          metrics: Specified product usage values

          product: Telnyx product

          date_range: A more user-friendly way to specify the timespan you want to filter by. More
              options can be found in the Telnyx API Reference docs.

          end_date: The end date for the time range you are interested in. The maximum time range is
              31 days. Format: YYYY-MM-DDTHH:mm:ssZ

          filter: Filter records on dimensions

          format: Specify the response format (csv or json). JSON is returned by default, even if
              not specified.

          managed_accounts: Return the aggregations for all Managed Accounts under the user making the
              request.

          sort: Specifies the sort order for results

          start_date: The start date for the time range you are interested in. The maximum time range
              is 31 days. Format: YYYY-MM-DDTHH:mm:ssZ

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/usage_reports",
            page=AsyncDefaultFlatPagination[UsageReportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dimensions": dimensions,
                        "metrics": metrics,
                        "product": product,
                        "date_range": date_range,
                        "end_date": end_date,
                        "filter": filter,
                        "format": format,
                        "managed_accounts": managed_accounts,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                        "start_date": start_date,
                    },
                    usage_report_list_params.UsageReportListParams,
                ),
            ),
            model=UsageReportListResponse,
        )

    async def get_options(
        self,
        *,
        product: str | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsageReportGetOptionsResponse:
        """
        Get the Usage Reports options for querying usage, including the products
        available and their respective metrics and dimensions

        Args:
          product: Options (dimensions and metrics) for a given product. If none specified, all
              products will be returned.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._get(
            "/usage_reports/options",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"product": product}, usage_report_get_options_params.UsageReportGetOptionsParams
                ),
            ),
            cast_to=UsageReportGetOptionsResponse,
        )


class UsageReportsResourceWithRawResponse:
    def __init__(self, usage_reports: UsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.list = to_raw_response_wrapper(
            usage_reports.list,
        )
        self.get_options = to_raw_response_wrapper(
            usage_reports.get_options,
        )


class AsyncUsageReportsResourceWithRawResponse:
    def __init__(self, usage_reports: AsyncUsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.list = async_to_raw_response_wrapper(
            usage_reports.list,
        )
        self.get_options = async_to_raw_response_wrapper(
            usage_reports.get_options,
        )


class UsageReportsResourceWithStreamingResponse:
    def __init__(self, usage_reports: UsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.list = to_streamed_response_wrapper(
            usage_reports.list,
        )
        self.get_options = to_streamed_response_wrapper(
            usage_reports.get_options,
        )


class AsyncUsageReportsResourceWithStreamingResponse:
    def __init__(self, usage_reports: AsyncUsageReportsResource) -> None:
        self._usage_reports = usage_reports

        self.list = async_to_streamed_response_wrapper(
            usage_reports.list,
        )
        self.get_options = async_to_streamed_response_wrapper(
            usage_reports.get_options,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action_hold_params import ActionHoldParams as ActionHoldParams
from .action_join_params import ActionJoinParams as ActionJoinParams
from .action_mute_params import ActionMuteParams as ActionMuteParams
from .action_play_params import ActionPlayParams as ActionPlayParams
from .action_stop_params import ActionStopParams as ActionStopParams
from .action_leave_params import ActionLeaveParams as ActionLeaveParams
from .action_speak_params import ActionSpeakParams as ActionSpeakParams
from .action_hold_response import ActionHoldResponse as ActionHoldResponse
from .action_join_response import ActionJoinResponse as ActionJoinResponse
from .action_mute_response import ActionMuteResponse as ActionMuteResponse
from .action_play_response import ActionPlayResponse as ActionPlayResponse
from .action_stop_response import ActionStopResponse as ActionStopResponse
from .action_unhold_params import ActionUnholdParams as ActionUnholdParams
from .action_unmute_params import ActionUnmuteParams as ActionUnmuteParams
from .action_update_params import ActionUpdateParams as ActionUpdateParams
from .action_leave_response import ActionLeaveResponse as ActionLeaveResponse
from .action_speak_response import ActionSpeakResponse as ActionSpeakResponse
from .action_unhold_response import ActionUnholdResponse as ActionUnholdResponse
from .action_unmute_response import ActionUnmuteResponse as ActionUnmuteResponse
from .action_update_response import ActionUpdateResponse as ActionUpdateResponse
from .action_send_dtmf_params import ActionSendDtmfParams as ActionSendDtmfParams
from .action_record_stop_params import ActionRecordStopParams as ActionRecordStopParams
from .action_send_dtmf_response import ActionSendDtmfResponse as ActionSendDtmfResponse
from .conference_command_result import ConferenceCommandResult as ConferenceCommandResult
from .action_record_pause_params import ActionRecordPauseParams as ActionRecordPauseParams
from .action_record_start_params import ActionRecordStartParams as ActionRecordStartParams
from .action_record_resume_params import ActionRecordResumeParams as ActionRecordResumeParams
from .action_record_stop_response import ActionRecordStopResponse as ActionRecordStopResponse
from .action_end_conference_params import ActionEndConferenceParams as ActionEndConferenceParams
from .action_record_pause_response import ActionRecordPauseResponse as ActionRecordPauseResponse
from .action_record_start_response import ActionRecordStartResponse as ActionRecordStartResponse
from .action_record_resume_response import ActionRecordResumeResponse as ActionRecordResumeResponse
from .action_end_conference_response import ActionEndConferenceResponse as ActionEndConferenceResponse
from .action_gather_dtmf_audio_params import ActionGatherDtmfAudioParams as ActionGatherDtmfAudioParams
from .action_gather_dtmf_audio_response import ActionGatherDtmfAudioResponse as ActionGatherDtmfAudioResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VoiceCloneCreateParams"]


class VoiceCloneCreateParams(TypedDict, total=False):
    gender: Required[Literal["male", "female", "neutral"]]
    """Gender of the voice clone."""

    language: Required[str]
    """ISO 639-1 language code for the clone (e.g. `en`, `fr`, `de`)."""

    name: Required[str]
    """Name for the voice clone."""

    voice_design_id: Required[str]
    """UUID of the source voice design to clone."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.ai import (
    EmbeddingResponse,
    EmbeddingListResponse,
    EmbeddingRetrieveResponse,
    EmbeddingSimilaritySearchResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEmbeddings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.create(
            bucket_name="bucket_name",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.create(
            bucket_name="bucket_name",
            document_chunk_overlap_size=0,
            document_chunk_size=0,
            embedding_model="thenlper/gte-large",
            loader="default",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.embeddings.with_raw_response.create(
            bucket_name="bucket_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = response.parse()
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.embeddings.with_streaming_response.create(
            bucket_name="bucket_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = response.parse()
            assert_matches_type(EmbeddingResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.retrieve(
            "task_id",
        )
        assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.embeddings.with_raw_response.retrieve(
            "task_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = response.parse()
        assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.embeddings.with_streaming_response.retrieve(
            "task_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = response.parse()
            assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            client.ai.embeddings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.list()
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.list(
            status=["string"],
        )
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.ai.embeddings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = response.parse()
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.ai.embeddings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = response.parse()
            assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_similarity_search(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.similarity_search(
            bucket_name="bucket_name",
            query="query",
        )
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_similarity_search_with_all_params(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.similarity_search(
            bucket_name="bucket_name",
            query="query",
            num_of_docs=0,
        )
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_similarity_search(self, client: Telnyx) -> None:
        response = client.ai.embeddings.with_raw_response.similarity_search(
            bucket_name="bucket_name",
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = response.parse()
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_similarity_search(self, client: Telnyx) -> None:
        with client.ai.embeddings.with_streaming_response.similarity_search(
            bucket_name="bucket_name",
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = response.parse()
            assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_url(self, client: Telnyx) -> None:
        embedding = client.ai.embeddings.url(
            bucket_name="bucket_name",
            url="url",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_url(self, client: Telnyx) -> None:
        response = client.ai.embeddings.with_raw_response.url(
            bucket_name="bucket_name",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = response.parse()
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_url(self, client: Telnyx) -> None:
        with client.ai.embeddings.with_streaming_response.url(
            bucket_name="bucket_name",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = response.parse()
            assert_matches_type(EmbeddingResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncEmbeddings:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.create(
            bucket_name="bucket_name",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.create(
            bucket_name="bucket_name",
            document_chunk_overlap_size=0,
            document_chunk_size=0,
            embedding_model="thenlper/gte-large",
            loader="default",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.embeddings.with_raw_response.create(
            bucket_name="bucket_name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = await response.parse()
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.embeddings.with_streaming_response.create(
            bucket_name="bucket_name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = await response.parse()
            assert_matches_type(EmbeddingResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.retrieve(
            "task_id",
        )
        assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.embeddings.with_raw_response.retrieve(
            "task_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = await response.parse()
        assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.embeddings.with_streaming_response.retrieve(
            "task_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = await response.parse()
            assert_matches_type(EmbeddingRetrieveResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `task_id` but received ''"):
            await async_client.ai.embeddings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.list()
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.list(
            status=["string"],
        )
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.embeddings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = await response.parse()
        assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.embeddings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = await response.parse()
            assert_matches_type(EmbeddingListResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_similarity_search(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.similarity_search(
            bucket_name="bucket_name",
            query="query",
        )
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_similarity_search_with_all_params(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.similarity_search(
            bucket_name="bucket_name",
            query="query",
            num_of_docs=0,
        )
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_similarity_search(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.embeddings.with_raw_response.similarity_search(
            bucket_name="bucket_name",
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = await response.parse()
        assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_similarity_search(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.embeddings.with_streaming_response.similarity_search(
            bucket_name="bucket_name",
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = await response.parse()
            assert_matches_type(EmbeddingSimilaritySearchResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_url(self, async_client: AsyncTelnyx) -> None:
        embedding = await async_client.ai.embeddings.url(
            bucket_name="bucket_name",
            url="url",
        )
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_url(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.embeddings.with_raw_response.url(
            bucket_name="bucket_name",
            url="url",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        embedding = await response.parse()
        assert_matches_type(EmbeddingResponse, embedding, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_url(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.embeddings.with_streaming_response.url(
            bucket_name="bucket_name",
            url="url",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            embedding = await response.parse()
            assert_matches_type(EmbeddingResponse, embedding, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AISummarizeParams"]


class AISummarizeParams(TypedDict, total=False):
    bucket: Required[str]
    """The name of the bucket that contains the file to be summarized."""

    filename: Required[str]
    """The name of the file to be summarized."""

    system_prompt: str
    """A system prompt to guide the summary generation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["EmbeddingListParams"]


class EmbeddingListParams(TypedDict, total=False):
    status: SequenceNotStr[str]
    """List of task statuses i.e. `status=queued&status=processing`"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["RunUpdateParams"]


class RunUpdateParams(TypedDict, total=False):
    mission_id: Required[str]

    error: str

    metadata: Dict[str, object]

    result_payload: Dict[str, object]

    result_summary: str

    status: Literal["pending", "running", "paused", "succeeded", "failed", "cancelled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .requests import (
    RequestsResource,
    AsyncRequestsResource,
    RequestsResourceWithRawResponse,
    AsyncRequestsResourceWithRawResponse,
    RequestsResourceWithStreamingResponse,
    AsyncRequestsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["VerificationResource", "AsyncVerificationResource"]


class VerificationResource(SyncAPIResource):
    @cached_property
    def requests(self) -> RequestsResource:
        """Manage your tollfree verification requests"""
        return RequestsResource(self._client)

    @cached_property
    def with_raw_response(self) -> VerificationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VerificationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VerificationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VerificationResourceWithStreamingResponse(self)


class AsyncVerificationResource(AsyncAPIResource):
    @cached_property
    def requests(self) -> AsyncRequestsResource:
        """Manage your tollfree verification requests"""
        return AsyncRequestsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncVerificationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVerificationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVerificationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVerificationResourceWithStreamingResponse(self)


class VerificationResourceWithRawResponse:
    def __init__(self, verification: VerificationResource) -> None:
        self._verification = verification

    @cached_property
    def requests(self) -> RequestsResourceWithRawResponse:
        """Manage your tollfree verification requests"""
        return RequestsResourceWithRawResponse(self._verification.requests)


class AsyncVerificationResourceWithRawResponse:
    def __init__(self, verification: AsyncVerificationResource) -> None:
        self._verification = verification

    @cached_property
    def requests(self) -> AsyncRequestsResourceWithRawResponse:
        """Manage your tollfree verification requests"""
        return AsyncRequestsResourceWithRawResponse(self._verification.requests)


class VerificationResourceWithStreamingResponse:
    def __init__(self, verification: VerificationResource) -> None:
        self._verification = verification

    @cached_property
    def requests(self) -> RequestsResourceWithStreamingResponse:
        """Manage your tollfree verification requests"""
        return RequestsResourceWithStreamingResponse(self._verification.requests)


class AsyncVerificationResourceWithStreamingResponse:
    def __init__(self, verification: AsyncVerificationResource) -> None:
        self._verification = verification

    @cached_property
    def requests(self) -> AsyncRequestsResourceWithStreamingResponse:
        """Manage your tollfree verification requests"""
        return AsyncRequestsResourceWithStreamingResponse(self._verification.requests)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.ai.missions.runs import plan_create_params, plan_update_step_params, plan_add_steps_to_plan_params
from .....types.ai.missions.runs.plan_create_response import PlanCreateResponse
from .....types.ai.missions.runs.plan_retrieve_response import PlanRetrieveResponse
from .....types.ai.missions.runs.plan_update_step_response import PlanUpdateStepResponse
from .....types.ai.missions.runs.plan_get_step_details_response import PlanGetStepDetailsResponse
from .....types.ai.missions.runs.plan_add_steps_to_plan_response import PlanAddStepsToPlanResponse

__all__ = ["PlanResource", "AsyncPlanResource"]


class PlanResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlanResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PlanResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlanResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PlanResourceWithStreamingResponse(self)

    def create(
        self,
        run_id: str,
        *,
        mission_id: str,
        steps: Iterable[plan_create_params.Step],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanCreateResponse:
        """
        Create the initial plan for a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan",
            body=maybe_transform({"steps": steps}, plan_create_params.PlanCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanCreateResponse,
        )

    def retrieve(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanRetrieveResponse:
        """
        Get the plan (all steps) for a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanRetrieveResponse,
        )

    def add_steps_to_plan(
        self,
        run_id: str,
        *,
        mission_id: str,
        steps: Iterable[plan_add_steps_to_plan_params.Step],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanAddStepsToPlanResponse:
        """
        Add one or more steps to an existing plan

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps",
            body=maybe_transform({"steps": steps}, plan_add_steps_to_plan_params.PlanAddStepsToPlanParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanAddStepsToPlanResponse,
        )

    def get_step_details(
        self,
        step_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanGetStepDetailsResponse:
        """
        Get details of a specific plan step

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGetStepDetailsResponse,
        )

    def update_step(
        self,
        step_id: str,
        *,
        mission_id: str,
        run_id: str,
        metadata: Dict[str, object] | Omit = omit,
        status: Literal["pending", "in_progress", "completed", "skipped", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanUpdateStepResponse:
        """
        Update the status of a plan step

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return self._patch(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps/{step_id}",
            body=maybe_transform(
                {
                    "metadata": metadata,
                    "status": status,
                },
                plan_update_step_params.PlanUpdateStepParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanUpdateStepResponse,
        )


class AsyncPlanResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlanResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlanResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlanResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPlanResourceWithStreamingResponse(self)

    async def create(
        self,
        run_id: str,
        *,
        mission_id: str,
        steps: Iterable[plan_create_params.Step],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanCreateResponse:
        """
        Create the initial plan for a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan",
            body=await async_maybe_transform({"steps": steps}, plan_create_params.PlanCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanCreateResponse,
        )

    async def retrieve(
        self,
        run_id: str,
        *,
        mission_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanRetrieveResponse:
        """
        Get the plan (all steps) for a run

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanRetrieveResponse,
        )

    async def add_steps_to_plan(
        self,
        run_id: str,
        *,
        mission_id: str,
        steps: Iterable[plan_add_steps_to_plan_params.Step],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanAddStepsToPlanResponse:
        """
        Add one or more steps to an existing plan

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps",
            body=await async_maybe_transform({"steps": steps}, plan_add_steps_to_plan_params.PlanAddStepsToPlanParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanAddStepsToPlanResponse,
        )

    async def get_step_details(
        self,
        step_id: str,
        *,
        mission_id: str,
        run_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanGetStepDetailsResponse:
        """
        Get details of a specific plan step

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._get(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps/{step_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanGetStepDetailsResponse,
        )

    async def update_step(
        self,
        step_id: str,
        *,
        mission_id: str,
        run_id: str,
        metadata: Dict[str, object] | Omit = omit,
        status: Literal["pending", "in_progress", "completed", "skipped", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlanUpdateStepResponse:
        """
        Update the status of a plan step

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not mission_id:
            raise ValueError(f"Expected a non-empty value for `mission_id` but received {mission_id!r}")
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        if not step_id:
            raise ValueError(f"Expected a non-empty value for `step_id` but received {step_id!r}")
        return await self._patch(
            f"/ai/missions/{mission_id}/runs/{run_id}/plan/steps/{step_id}",
            body=await async_maybe_transform(
                {
                    "metadata": metadata,
                    "status": status,
                },
                plan_update_step_params.PlanUpdateStepParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlanUpdateStepResponse,
        )


class PlanResourceWithRawResponse:
    def __init__(self, plan: PlanResource) -> None:
        self._plan = plan

        self.create = to_raw_response_wrapper(
            plan.create,
        )
        self.retrieve = to_raw_response_wrapper(
            plan.retrieve,
        )
        self.add_steps_to_plan = to_raw_response_wrapper(
            plan.add_steps_to_plan,
        )
        self.get_step_details = to_raw_response_wrapper(
            plan.get_step_details,
        )
        self.update_step = to_raw_response_wrapper(
            plan.update_step,
        )


class AsyncPlanResourceWithRawResponse:
    def __init__(self, plan: AsyncPlanResource) -> None:
        self._plan = plan

        self.create = async_to_raw_response_wrapper(
            plan.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            plan.retrieve,
        )
        self.add_steps_to_plan = async_to_raw_response_wrapper(
            plan.add_steps_to_plan,
        )
        self.get_step_details = async_to_raw_response_wrapper(
            plan.get_step_details,
        )
        self.update_step = async_to_raw_response_wrapper(
            plan.update_step,
        )


class PlanResourceWithStreamingResponse:
    def __init__(self, plan: PlanResource) -> None:
        self._plan = plan

        self.create = to_streamed_response_wrapper(
            plan.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            plan.retrieve,
        )
        self.add_steps_to_plan = to_streamed_response_wrapper(
            plan.add_steps_to_plan,
        )
        self.get_step_details = to_streamed_response_wrapper(
            plan.get_step_details,
        )
        self.update_step = to_streamed_response_wrapper(
            plan.update_step,
        )


class AsyncPlanResourceWithStreamingResponse:
    def __init__(self, plan: AsyncPlanResource) -> None:
        self._plan = plan

        self.create = async_to_streamed_response_wrapper(
            plan.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            plan.retrieve,
        )
        self.add_steps_to_plan = async_to_streamed_response_wrapper(
            plan.add_steps_to_plan,
        )
        self.get_step_details = async_to_streamed_response_wrapper(
            plan.get_step_details,
        )
        self.update_step = async_to_streamed_response_wrapper(
            plan.update_step,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .s3_configuration_data import S3ConfigurationData
from .gcs_configuration_data import GcsConfigurationData
from .azure_configuration_data import AzureConfigurationData

__all__ = ["CustomStorageConfiguration", "Configuration"]

Configuration: TypeAlias = Annotated[
    Union[GcsConfigurationData, S3ConfigurationData, AzureConfigurationData], PropertyInfo(discriminator="backend")
]


class CustomStorageConfiguration(BaseModel):
    backend: Literal["gcs", "s3", "azure"]

    configuration: Configuration



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, TypedDict

__all__ = ["WhatsappMessageTemplateUpdateParams"]


class WhatsappMessageTemplateUpdateParams(TypedDict, total=False):
    category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"]

    components: Iterable[Dict[str, object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["HangupToolParams"]


class HangupToolParams(BaseModel):
    description: Optional[str] = None
    """The description of the function that will be passed to the assistant."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .ssl_certificate import SslCertificate

__all__ = ["SslCertificateCreateResponse"]


class SslCertificateCreateResponse(BaseModel):
    data: Optional[SslCertificate] = None
